/* This example sketch illustrates how to use the Azatrax library
 * functions to communicate with the IR detector circuits of an
 * RIR4 shield.
 * 
 * Written by John Parsons for public use.  No warranty.
 * revised:
 * 10 May 2018 Initial release.
 */

#include <Azatrax.h>    // enables access to the Azatrax library functions

Azatrax RIR4(0x38); // Create RIR4 as an instance of class Azatrax.
                    // This is the RIR4 circuit with I2C address set to 0x38.
// If more than one RIR4 is being used, give each one a unique name
// and set the I2C address selection switches to unique values for each RIR4.
// Initialize each RIR4 (create an instance of class Azatrax for each circuit).
// For example,
//   Azatrax RIR41(0x30); // first RIR4, addr switch set for 0x30
//   Azatrax RIR42(0x32); // second RIR4, addr switch set for 0x32
//   Azatrax RIR43(0x34); // third RIR4, addr switch set for 0x34
//
// Then, where you find RIR4 in the following code, substitute RIR41, RIR42 or
// RIR43 in place of RIR4.

// Prototype function declaration
// This function is found at the end of this sketch.  This declaration
// informs the compiler of the function's existence.
int keyedNumericInput(void);

void setup() {
    Serial.begin(9600);   // to use the serial monitor
}

void loop() {
    static byte i2cAddr;
    static byte detNum;
    static byte devData;
    static byte newThr;

    Serial.println();
    // Read the contents of RIR4's internal register 0 (detector state bitmap).
    // The upper four bits should all be 0.
    devData = RIR4.getDetData(0x00);
    if(0xFF == devData) {
        // If an unexpected value was received, then...
        i2cAddr = (byte)RIR4.devAddr;   // Get the address that was used when this
                        // RIR4 was initialized at the top of this sketch.
                        // Convert it to byte format.
        Serial.print("Device at address 0x");
        Serial.print(i2cAddr, HEX);     // print in hexadecimal format
        Serial.println(" did not respond as expected.");
    }
    else {
        // A good reply was received...
        Serial.print("Detector state bitmap: ");
        Serial.println(devData, BIN);   // print in binary format

        // Using the boolean functions:
        Serial.print("Detector 4:");
        if(true == RIR4.detOccupied(4)) {   // this function returns 'true' when the
            Serial.print("occupied");       // specified detector is sensing an object
        }
        else {
            Serial.print("vacant  ");
        }
        Serial.print("  3:");
        if(true == RIR4.detOccupied(3)) {   // this function returns 'true' when the
            Serial.print("occupied");       // specified detector is sensing an object
        }
        else {
            Serial.print("vacant  ");
        }
        Serial.print("  2:");
        if(true == RIR4.detVacant(2)) {  // this function returns 'true' when the
            Serial.print("vacant  ");  // specified detector is not sensing an object
        }
        else {
            Serial.print("occupied");
        }
        Serial.print("  1:");
        if(true == RIR4.detVacant(1)) {  // this function returns 'true' when the
            Serial.println("vacant");    // specified detector is not sensing an object
        }
        else {
            Serial.println("occupied");
        }
    
        // Get and display the individual IR signal levels for detectors 1-4
        Serial.print("    Signal levels");
        for(detNum = 4; detNum > 0; detNum--) {
            // detNum is the detector #, counting from 4 down to 1
            devData = RIR4.getSignalLevel(detNum);
            Serial.print(",  #");
            Serial.print(detNum);
            Serial.print(": ");
            Serial.print(devData);  // print in decimal format
        }
        Serial.println();
    
        // Get and display the detector threshold levels.
        // Each detector's signal level must exceed the ambient IR
        // level ("background noise") by at least this amount to
        // change the detector status from "off" to "on."
        Serial.print(" Threshold levels");
        for(detNum = 4; detNum > 0; detNum--) {
            devData = RIR4.getThreshold(detNum);
            Serial.print(",  #");
            Serial.print(detNum);
            Serial.print(": ");
            Serial.print(devData);
        }
        Serial.println();
    
        // Get and display ambient IR level for each detector.
        Serial.print("Ambient IR levels");
        for(detNum = 4; detNum > 0; detNum--) {
            devData = RIR4.getAmbient(detNum);
            Serial.print(",  #");
            Serial.print(detNum);
            Serial.print(": ");
            Serial.print(devData);
        }
        Serial.println();
    }
    // Give the user an opportunity to change a detector's threshold.
    // Sending a value of 1-254 will set the detector's threshold to that value.
    // Sending a value of 0 or 255 will reset the threshold to the factory default.
    Serial.println("To change a detector's threshold,");
    Serial.println("enter the detector's number (1-4) above.");
    Serial.println("   Or enter any other character to skip.");
    detNum = (byte)keyedNumericInput();
    if((1 <= detNum) && (detNum <= 4)) {
        Serial.print("Enter new threshold value (0-255) for detector ");
        Serial.println(detNum);

        newThr = (byte)keyedNumericInput();

        RIR4.setThreshold(detNum, newThr);  // send to the RIR4 circuit
        
        Serial.print("   New threshold value = ");
        Serial.println(newThr);
    }
    else {
        Serial.println("   skipping");
    }
}

/***********************************
 * keyedNumericInput
 * This routine accepts serial input, such as from the serial monitor,
 * decodes the ASCII characters and returns an integer.
 *
 ***********************************/
int keyedNumericInput()
{
    String inString;    // string to hold the ASCII input
    char inChar;        // holds the next character from input

    inString = "";
    while(Serial.available() == 0) {
        // Wait for user input
    }
    while(Serial.available() > 0) {
        // Build a string out of the entered characters that are numeric
        // digits (between '0' and '9').
        inChar = (char)Serial.read();
        if(('0' <= inChar) && (inChar <= '9')) {
            inString.concat(inChar);   // appends inChar to the end of inString
        }
        delay(10);  // Allow next character to enter the buffer.
                    // Without this delay, the loop sometimes exits before
                    // the entire input string is processed.
    }
    return inString.toInt();
}

